unit FloatEdit;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

uses
{$IFnDEF FPC}
  Windows,
{$ELSE}
  LCLIntf, LCLType, LMessages,
{$ENDIF}
  Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;


type
  TFloatEdit = class(TCustomEdit)
  private
    { Private-Deklarationen }
    FNk : integer;
    FMaxLength : integer;
    FMinValue : double;
    FMaxValue : double;
    FValue : double;
    procedure SetNk(Value : integer);
    procedure SetMinValue(Value : double);
    procedure SetMaxValue(Value : double);
    procedure SetValue(Val : double);
    procedure SetMaxLength(Value : integer);
  protected
    { Protected-Deklarationen }
    procedure Change; override;
    procedure KeyPress(var Key: Char); override;
    procedure DoEnter; override;
    procedure DoExit; override;
  public
    { Public-Deklarationen }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published-Deklarationen }
    property NK : integer read FNk write SetNK default 2;
    property MaxLength : integer read FMaxLength write SetMaxLength default 8;
    property MinValue : double read FMinValue write SetMinValue;
    property MaxValue : double read FMaxValue write SetMaxValue;
    property Value : double read FValue write SetValue;
    property AutoSelect;
    property AutoSize;
    property BorderStyle;
    property Color;
   { property CTl3D;  }
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property HelpContext;
    property HideSelection;
    property ParentColor;
    {property ParentCtl3D;}
    property ParentFont;
    property ParentShowHint;
    property ReadOnly;
    property ShowHint;
    property TabOrder;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
  end;

(*
Wenn Du Deine Komponente von tCustomEdit ableitest dann kannst Du
die Dynamisch deklarierten Methoden DoEnter und DoExit überschreiben.
In der Deklaration musst Du dann halt Deine oben erwähnte Mehtode X aufrufen
und dann halt noch Inherited damit das vom User definierte Ereignis aufgerufen wird.

HighBam

*)

procedure Register;

implementation

{$R TFloatEdit.lrs }


const
  FloatKeys : set of char =['0','1','2','3','4','5','6','7','8','9','-','+','.',#8];

constructor TFloatEdit.Create(AOwner: TComponent);
var
  s :string;

begin
  inherited Create(AOwner);
  FNk:=2;
  FMaxLength:=11;
  FMinValue:=-1200;
  FMaxValue:=1000;
  FValue:=0;
  Str(FValue:MaxLength:nk,s);
  Text:=s;
end;

destructor TFloatEdit.Destroy;
begin
  inherited Destroy;
end;

procedure TFloatEdit.Change;
var
  TempValue : double;
  TextNeu : string;

begin
  if Focused then begin
    try
      TextNeu:=Trim(Text);
     TempValue:=StrToFloat(TextNeu);
      if TempValue<MinValue then begin
        TempValue:=MinValue;
      end;
      if TempValue>MaxValue then begin
        TempValue:=MaxValue;
      end;
      FValue:=TempValue;
    except
    end;
  end;
  inherited Change;
end;

procedure TFloatEdit.KeyPress(var Key: Char);
var
  TextoM : string; { Text ohne Markierung }
  pp,pmi,ppl: integer; { Position '.','-','+' }

begin
  inherited KeyPress(Key);
  TextoM:=Copy(Text,1,SelStart)+Copy(Text,Selstart+SelLength+1,Length(Text)); { Text ohne Markierung }
  if (Key in FloatKeys) and not ReadOnly and (Length(TextoM)<MaxLength) then begin
    pp:=Pos('.',TextoM); { Pos Punkt }
    pmi:=Pos('-',TextoM); { Pos Minus }
    ppl:=Pos('+',TextoM); { Pos Plus }
    case Key of
      '0'..'9':begin
        if (SelStart>=pp) and (Length(TextoM)-pp>=NK) and (pp>0) then key:=#0;
      end;
      '.': begin
        if pp>0 then Key:=#0;
      end;
      '-','+': begin
        if (SelStart>0) or (pmi>0) or (ppl>0) then Key:=#0;
      end;
      #8: begin
      end;
    end; { case }
  end
  else Key:=#0;
end;

procedure TFloatEdit.DoEnter;
begin
  inherited;
  Text:=Trim(Text);
  if AutoSelect then SelectAll;
end;

procedure TFloatEdit.DoExit;
var
  s : string;
begin
  inherited;
  Str(FValue:MaxLength:nk,s);
  Text:=s;
end;

procedure TFloatEdit.SetNK(Value : integer);
var
  s : string;
begin
  if FNk<>Value then begin
    FNk:=Value;  { Nachkommastellen }
    if  not Focused then begin
      Str(FValue:MaxLength:nk,s);
      Text:=s;
    end;
  end;
end;

procedure TFloatEdit.SetMaxLength(Value : integer);
var
  s : string;
begin
  if FMaxLength<>Value then begin
    FMaxLength:=Value;  { Nachkommastellen }
    if not Focused then begin
      Str(FValue:MaxLength:nk,s);
      Text:=s;
    end;
  end;
end;

procedure TFloatEdit.SetValue(Val : double);
var
  s : string;
begin
  if FValue<>Val then begin
    if Val>FMaxValue then FValue:=FMaxValue
    else if Val<FMinValue then FValue:=FMinValue else FValue:=Val;  { Wert }
    if not focused then begin
      Str(FValue:MaxLength:nk,s);
      Text:=s;
    end;
  end;
end;

procedure TFloatEdit.SetMaxValue(Value : double);
begin
  FMaxValue:=Value;  { Maximalwert }
end;

procedure TFloatEdit.SetMinValue(Value : double);
begin
  FMinValue:=Value;  { Minimalwert }
end;

procedure Register;
begin
  RegisterComponents('Beispiele', [TFloatEdit]);
end;

end.
