unit Meter120;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

(************** Messinstrument mit 120-Grad-Skala ********************)
interface

uses
{$IFnDEF FPC}
  Windows,
{$ELSE}
  LCLIntf, LCLType, LMessages,
{$ENDIF}
  Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, Windows;


type
  TVorsatz = (Atto,Femto,Pico,Nano,Mikro,Milli,kein,Kilo,Mega,Giga,Tera,Peta,Exa);
  TBereich = (Ber1,Ber2,Ber5, Ber10, Ber20, Ber50, Ber100, Ber200, Ber500);
  TOnChange = procedure(Sender: TObject;AktWert:Double) of object;
  {TSize = record
    cx,cy:integer;
  end;   }

const
  VorsatzFaktor : array[Atto..Exa] of double = (1e-18,1e-15,1e-12,1e-9,1e-6,
                                                 1e-3,1,1e3,1e6,1e9,1e12,
                                                 1e15,1e18);
  VorsatzBuchstabe : array[Atto..Exa] of string[1] =('a','f','p','n','','m','','k','M','G','T','P','E');
  BereichFaktor : array[Ber1..Ber500] of double = (1,2,5,10,20,50,100,200,500);

type
  TMeter120 = class(TGraphicControl)
  private
    { Private-Deklarationen }
    FOnChange: TOnChange;
    FPen : TPen;      { Kreis + Teilstriche }
    FBrush : TBrush;  { Hintergrung }
    FZeigerfarbe : TColor; { Zeigerfarbe }
    FFont : TFont;    { Fr Skalawerte }
    FWert : Double;{ Dieser Wert wird angezeigt }
    FBereich : TBereich; { 0.1 ... 50 }
    FVorsatz : TVorsatz; { Atto .. Exa }
    FEinheit : String; { z.B. V fr Volt, Ps fr Pascal etc. }
    FAutoRange : boolean; { Whlt Bereich und Vorsatz automatisch }
    FWertAnzeigen : boolean; { Zahlenwert anzeigen }
    FRundeEcken : boolean;
    procedure SetBrush(Value : TBrush);
    procedure SetZeigerfarbe(Value : TColor);
    procedure SetWert(Value : Double);
    procedure SetBereich(Value : TBereich);
    procedure SetVorsatz(Value : TVorsatz);
    procedure SetEinheit(Value : String);
    procedure SetPen(Value : TPen);
    procedure SetFont(Value : TFont);
    procedure SetAutoRange(Value : boolean);
    procedure SetWertAnzeigen(Value : boolean);
    procedure SetRundeEcken(Value : boolean);
  protected
    { Protected-Deklarationen }
    procedure Paint; override;
    procedure DoOnChange(Sender: TObject;AktWert:Double);dynamic;
  public
    { Public-Deklarationen }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published-Deklarationen }
    property OnChange: TOnChange read FOnChange write FOnChange;
    property Width;
    property Brush: TBrush read FBrush write SetBrush;
    property Zeigerfarbe: TColor read FZeigerfarbe write SetZeigerfarbe;
    property Wert : Double read FWert write SetWert;
    property Bereich : TBereich read FBereich write SetBereich;
    property Vorsatz : TVorsatz read FVorsatz write SetVorsatz;
    property Einheit : String read FEinheit write SetEinheit;
    property Pen: TPen read FPen write SetPen;
    property Font: TFont read FFont write SetFont;
    property Autorange : boolean read FAutorange write SetAutorange;
    property WertAnzeigen : boolean read FWertAnzeigen write SetWertAnzeigen;
    property RundeEcken : boolean read FRundeEcken write SetRundeEcken;
    property DragCursor;
    property DragMode;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    procedure StyleChanged(Sender : TObject);
  end;

procedure Register;

implementation
(*
{$R Meter120.lrs }
*)

(* Methoden fr TMeter120 =========================================*)

procedure TMeter120.DoOnChange(Sender: TObject;AktWert:Double);
begin
  if Assigned(OnChange) then OnChange(Self,AktWert);
end;

function Schattenfarbe(c : TColor): TColor;
var
  r,b,g : byte;
begin
  { Schattenfarbe bestimmen }
  r:=LongInt(c) and $FF;
  g:=(LongInt(c) and $FF00) shr 8;
  b:=(LongInt(c) and $FF0000) shr 16;

  { Alle Farbkompoenten ein bisschen dunkler=kleiner }
  {r:=r*20 div 28;
  b:=b*20 div 28;
  g:=g*20 div 28;}
  if r>64 then r:=r-64 else r:=0;
  if b>64 then b:=b-64 else b:=0;
  if g>64 then g:=g-64 else g:=0;

  Schattenfarbe:=TColor((b shl 16)+(g shl 8)+r); { Schattenfarbe }
end;

function Lg(x:double):double;
begin
  lg:=ln(x)/ln(10)
end;

constructor TMeter120.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Self.Width:=256;
  Self.Height:=Width * 2 div 3;
  FBrush:=TBrush.Create;  { Hintergrund }
  FBrush.OnChange:=StyleChanged;
  FFont:=TFont.Create;
  FFont.Size:=Self.Width div 20;
  FFont.OnChange:=StyleChanged;
  FPen:=TPen.Create;
  FPen.OnChange:=StyleChanged;
  Fwert:=0;
end;

destructor TMeter120.Destroy;
begin
  FPen.Free;
  inherited Destroy;
end;

procedure TMeter120.Paint;
const
  bm = pi/180;
  ds = 2;

var
  xm,ym,r,rz,w,po,tsl, wi,zl, vsi,nk : integer;
  tswert, dts, endwert: real;
  x,y,n,nl,nm :integer;
  ts : TSize;
  strwmi, strwmax : string[5];

  normWert,wb : double;
  bstr,vz : string[20]; { Fr Bereichstext }
  vsn: TVorsatz;
  b : TBereich;

procedure ZeigZeiger(Wi,lz,db,ds:integer;col:TColor);
var
  pts : ARRAY[0..2] of TPoint;

begin { ZeigZeiger }
  with Canvas do begin
  Brush.Color:=col;
  Pen.Color:=col;
  pts[0].X:=xm-round(lz*cos(bm*wi))+ds; { Spitze}
  pts[0].Y:=ym-round(lz*sin(bm*wi))+ds;
  pts[1].X:=xm-round(db*cos(bm*(wi+90)))+ds; { linke Ecke }
  pts[1].Y:=ym-round(db*sin(bm*(wi+90)))+ds;
  pts[2].X:=xm-round(db*cos(bm*(wi-90)))+ds; { rechte Ecke }
  pts[2].Y:=ym-round(db*sin(bm*(wi-90)))+ds;
  Polygon(pts);
  Ellipse(xm+rz,ym+rz,xm-rz,ym-rz);
  end;
end;


begin { Paint }
  with Canvas do begin
    Pen:=FPen;     { Kreis }
    Brush:=FBrush; { Hintergrund }
    Font:=FFont;
    xm:=Self.Width div 2;
    ym:=Self.Width *3 div 5;
    po:=pen.Width div 2;  { Penoffset, halbe Pen-Breite }
    if FRundeEcken then roundrect(po,po,Self.width-po,Self.Height-po,Self.Width div 7,Self.Width div 7)
    else rectangle(po,po,Self.width-po,Self.Height-po);
    r:=Self.Width*6 div 14;  { Radius des Kreises }
    rz:=r div 20;     { Kreischen im Zeigerdrehpunkt }
    zl:=r*9 div 10;   { Zeigerlnge }
    Arc(xm-r,ym-r,xm+r,ym+r,xm+r,ym-r*577 div 1000,xm-r,ym-r*577 div 1000);{ Zeigerzentrum }

    { Zeiger Neuer Wert anzeigen }
    if FAutorange and (FWert<>0)then begin { Autorange }
      vsi:=trunc(Lg(abs(FWert)*2)+18) div 3; { Vorsatzindex }
      if (vsi>=0)  and (vsi<=ord(Exa)) then begin { in gltigem Bereich }
        vsn:=TVorsatz(vsi); { Vorsatz des neuen Werts}
        if vsn<>Vorsatz then Vorsatz:=vsn;
        wb:=abs(FWert)/VorsatzFaktor[vsn];
        b:=Ber1;
        if  wb>1 then b:=Ber2;
        if  wb>2 then b:=Ber5;
        if  wb>5 then b:=Ber10;
        if  wb>10 then b:=Ber20;
        if  wb>20 then b:=Ber50;
        if  wb>50 then b:=Ber100;
        if  wb>100 then b:=Ber200;
        if  wb>200 then b:=Ber500;
        if b<>FBereich then
         FBereich:=b;
      end;
    end; { Autorange }
    normWert:=abs(FWert/BereichFaktor[FBereich]/VorsatzFaktor[FVorsatz]); { Auf Max. Ausschlag normalisierter Wert }
    if normWert>1.05 then NormWert:=1.05; { Begrenzen }

    { Bereich anzeigen }
    Pen:=FPen;
    vz:=' '; { positiv: kein Vorzeichen }
    if  FWert<0 then vz:='-';
    bstr:=vz+FloatToStr(BereichFaktor[FBereich])+' '+VorsatzBuchstabe[FVorsatz]+FEinheit;
    ts:=TextExtent(bstr);
    TextOut(xm-ts.cx div 2,ym-r+ts.cy ,bstr);

    { Skalawerte anschreiben }
    case FBereich of
      Ber1: begin strwmi:='0.5'; strwmax:='1'; end;
      Ber10: begin strwmi:='5'; strwmax:='10'; end;
      Ber100: begin strwmi:='50'; strwmax:='100'; end;
      Ber2: begin strwmi:='1'; strwmax:='2'; end;
      Ber20: begin strwmi:='10'; strwmax:='20'; end;
      Ber200: begin strwmi:='100'; strwmax:='200'; end;
      Ber5: begin strwmi:='2.5'; strwmax:='5'; end;
      Ber50: begin strwmi:='25'; strwmax:='50'; end;
      Ber500: begin strwmi:='250'; strwmax:='500'; end;
    end; { case }                 { Skalawert 0 }
    ts:=TextExtent('0');
    TextOut(xm-r*10 div 9,r*10 div 12-ts.cy,'0');
    ts:=TextExtent(strwmi);       { Mittlere Skalawert }
    tsl:=r div 7;     { langer Teilstrich }
    TextOut(xm-ts.cx div 2,ym-r-tsl-ts.cy,strwmi);
    ts:=TextExtent(strwmax);      { Skalawert max }
    TextOut(Self.width-ts.cx-4,r*10 div 12-ts.cy,strwmax);

    { Teilstriche zeichnen }
    tswert:=0;n:=0;
    case (ord(FBereich) mod 3) of
      0: begin dts:=0.05; endwert:=1;nl:=round(endwert/dts) div 2;nm:=2; end;
      1: begin dts:=0.1; endwert:=2;nl:=round(endwert/dts) div 2;nm:=5; end;
      2: begin dts:=0.2; endwert:=5;nl:=round(endwert/dts)div 5;nm:=100; end;
    end; { case }
    while tswert<=(endwert + dts/2) do begin
      tsl:=r div 18;     { kurzer Teilstrich }
      if (n mod nm) = 0 then  tsl:=r div 12; { Langer Teilstrich }
      if (n mod nl) = 0 then  tsl:=r div 7; { Langer Teilstrich }
      w:=round(tswert/endwert*120)+30;
      y:=round((r+tsl)*sin(bm*w));x:=round((r+tsl)*cos(bm*w));
      MoveTo(xm-x,ym-y);
      y:=round(r*sin(bm*w));x:=round(r*cos(bm*w));
      LineTo(xm-x,ym-y);
      tswert:=tswert+dts;
      inc(n) { nchster Teilstrich }
    end;

    { Wert anzeigen }
    nk:=0; { Anzahl Nachkommastellen, 0 wenn Zahl >=1000 }
    {normwert:=abs(FWert)/VorsatzFaktor[FVorsatz];}
    if (normWert>0) and (lg(normwert)<3) then
      nk:=trunc(3-lg(normwert));
    bstr:=FloatToStrF(FWert/VorsatzFaktor[FVorsatz],ffNumber,7,nk)+' '
      +VorsatzBuchstabe[FVorsatz]+FEinheit;
    ts:=TextExtent(bstr);
    pen.Color:=Brush.Color;
    rectangle(xm-r,ym-ts.cy,xm+r,ym); { alter Wert berschreiben }
    if FWertAnzeigen then TextOut(xm-r,ym-ts.cy,bstr);


    { Zeiger anzeigen }
    wi:=30+round(120*normWert);
    ZeigZeiger(wi,zl,3,ds,Schattenfarbe(FBrush.Color));
    ZeigZeiger(wi,zl,3,0,FZeigerfarbe);
    DoOnChange(Self,FWert);
  end; { with }
end;

procedure TMeter120.SetWert(Value : Double);

begin { SetWert }
  if FWert<>Value then begin
    FWert:=Value;  { Wert }
    Invalidate;
  end;
end;

procedure TMeter120.SetZeigerfarbe(Value : TColor);
begin
  if FZeigerfarbe<>Value then begin
    FZeigerfarbe:=Value;  { Farbe des Zeigers }
    Invalidate;
  end;
end;

procedure TMeter120.SetBrush(Value : TBrush);
begin
  if FBrush<>Value then begin
    FBrush:=Value;  { Hintergrund }
    Invalidate;
  end;
end;

procedure TMeter120.SetFont(Value : TFont);
begin
  if FFont<>Value then begin
    FFont:=Value;
    Invalidate;
  end;
end;

procedure TMeter120.SetPen(Value : TPen);
begin
  if FPen<>Value then begin
    FPen:=Value;
    Invalidate;
  end;
end;

procedure TMeter120.SetBereich(Value : TBereich);
begin
  if FBereich<>Value then begin
    FBereich:=Value;
    invalidate;
  end;
end;

procedure TMeter120.SetVorsatz(Value : TVorsatz);
begin
  if FVorsatz<>Value then begin
    FVorsatz:=Value;
    invalidate;
  end;
end;

procedure TMeter120.SetEinheit(Value : String);
begin
  if FEinheit<>Value then begin
    FEinheit:=Value;
    invalidate;
  end;
end;

procedure TMeter120.SetWertAnzeigen(Value : boolean);
begin
  if FWertAnzeigen<>Value then begin
    FWertAnzeigen:=Value;
    invalidate;
  end;
end;

procedure TMeter120.SetAutoRange(Value : boolean);
begin
  if FAutoRange<>Value then begin
    FAutoRange:=Value;
    invalidate;
  end;
end;

procedure TMeter120.SetRundeEcken(Value : boolean);
begin
  if FRundeEcken<>Value then begin
    FRundeEcken:=Value;
    invalidate;
  end;
end;

procedure TMeter120.StyleChanged(Sender : TObject);
begin
  Invalidate;
end;

procedure Register;
begin
  RegisterComponents('Beispiele', [TMeter120]);
end;

end.
