(* Komponente Schalter, Linie und Dreieck selbst entwickelt *)

unit Tline;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

uses
{$IFnDEF FPC}
  WinTypes, WinProcs,
{$ELSE}
  LCLIntf, LCLType, LMessages,
{$ENDIF}
  SysUtils, Messages, Classes, Graphics, Controls, Windows,
  Forms, Dialogs, ExtCtrls;


type
  LineDirectionType = (Hori,Verti,LoRu,LuRo);
  TXLinie = class(TGraphicControl)
  private
    { Private-Deklarationen }
    LineDirection : LineDirectionType;
    FPen : TPen;
    procedure SetDirection(Value : LineDirectionType);
    procedure SetPen(Value : TPen);
  protected
    { Protected-Deklarationen }
    procedure Paint; override;
  public
    { Public-Deklarationen }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published-Deklarationen }
    property Direction : LineDirectionType
      read LineDirection write SetDirection;
    property Height default 50;
    property Width default 50;
    property Pen: TPen read FPen write SetPen;
    property DragCursor;
    property DragMode;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    procedure StyleChanged(Sender : TObject);
  end;

  SpitzenSeiteType = (Rechts,Links,Oben,Unten);
  TDreieck = class(TGraphicControl)
  private
    { Private-Deklarationen }
    FSpitzenSeite : SpitzenSeiteType;
    FSpitze : integer;
    FPen : TPen;
    FBrush : TBrush;
    procedure SetSpitzenSeite(Value : SpitzenSeiteType);
    procedure SetSpitze(Value : integer);
    procedure SetPen(Value : TPen);
    procedure SetBrush(Value : TBrush);
  protected
    { Protected-Deklarationen }
    procedure Paint; override;
  public
    { Public-Deklarationen }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published-Deklarationen }
    property SpitzenSeite : SpitzenSeiteType
      read FSpitzenSeite write SetSpitzenSeite;
    property Height default 50;
    property Width default 50;
    property Spitze : integer read FSpitze write SetSpitze;
    property Brush: TBrush read FBrush write SetBrush;
    property Pen: TPen read FPen write SetPen;
    property DragCursor;
    property DragMode;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    procedure StyleChanged(Sender : TObject);
  end;

  { Schalter }
  DrehpunktSeiteType = (DPRechts,DPLinks,DPOben,DPUnten);
  TSchalter = class(TGraphicControl)
  private
    { Private-Deklarationen }
    FDrehpunktSeite : DrehpunktSeiteType;
    FPen : TPen;
    FBrush : TBrush;
    FOeffnenNachLinks : boolean;
    FState : boolean;
    FOnChange: TNotifyEvent;
    procedure SetDrehpunktSeite(Value : DrehpunktSeiteType);
    procedure SetOeffnenNachLinks(Value : Boolean);
    procedure SetPen(Value : TPen);
    procedure SetBrush(Value : TBrush);
    procedure SetState(Value : boolean);
  protected
    { Protected-Deklarationen }
    procedure Paint; override;
    procedure Change; dynamic;
  public
    { Public-Deklarationen }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published-Deklarationen }
    property DrehpunktSeite : DrehpunktSeiteType
      read FDrehpunktSeite write SetDrehpunktSeite;
    property Width  default 50;
    property Brush: TBrush read FBrush write SetBrush;
    property Pen: TPen read FPen write SetPen;
    property OeffnenNachLinks:boolean
      read FOeffnenNachLinks write SetOeffnenNachLinks;
    property State : boolean read FState write SetState;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property DragCursor;
    property DragMode;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnClick;
    procedure StyleChanged(Sender : TObject);
  end;

  { Umschalter }
  TUmschalter = class(TGraphicControl)
  private
    { Private-Deklarationen }
    FDrehpunktSeite : DrehpunktSeiteType;
    FPen : TPen;
    FBrush : TBrush;
    FState : boolean;
    FOnChange: TNotifyEvent;
    procedure SetDrehpunktSeite(Value : DrehpunktSeiteType);
    procedure SetPen(Value : TPen);
    procedure SetBrush(Value : TBrush);
    procedure SetState(Value : boolean);
  protected
    { Protected-Deklarationen }
    procedure Paint; override;
    procedure Change; dynamic;
  public
    { Public-Deklarationen }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published-Deklarationen }
    property DrehpunktSeite : DrehpunktSeiteType
      read FDrehpunktSeite write SetDrehpunktSeite;
    property Width  default 50;
    property Brush: TBrush read FBrush write SetBrush;
    property Pen: TPen read FPen write SetPen;
    property State : boolean read FState write SetState;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property DragCursor;
    property DragMode;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnClick;
    procedure StyleChanged(Sender : TObject);
  end;

procedure Register;


implementation

(*
{$R TXLine.lrs }
{$R TDreieck.lrs }
{$R TSchalter.lrs }
{$R TUmschalter.lrs }
*)

procedure Register;
begin
  RegisterComponents('Beispiele', [TXLinie]);
  RegisterComponents('Beispiele', [TDreieck]);
  RegisterComponents('Beispiele', [TSchalter]);
  RegisterComponents('Beispiele', [TUmschalter]);
end;


(* Methoden für TLine =========================================*)
constructor TXLinie.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Self.Width:= 50;
  Self.Height:= 50;
  FPen:=TPen.Create;
  FPen.Color :=clBlack;
  FPen.OnChange:=StyleChanged;
end;

destructor TXLinie.Destroy;
begin
  FPen.Free;
  inherited Destroy;
end;

procedure TXLinie.Paint;
var
  pw,pwh : INTEGER;

  begin
  with Canvas do begin
    Pen:=FPen;
    pw:=Pen.Width;
    pwh:=pw div 2;
    if pwh=0 then pwh:=1;
    if Self.Width<pw THEN Self.Width:=pw;
    if Self.Height<pw THEN Self.Height:=pw;
    case LineDirection of
      Hori: begin MoveTo(pwh,pwh);LineTo(Self.Width-pwh,pwh); end;
      Verti:begin MoveTo(pwh,pwh);LineTo(pwh,Self.Height-pwh); end;
      LoRu: begin MoveTo(pwh,pwh);LineTo(Self.Width-pwh,Self.Height-pwh); end;
      LuRo: begin MoveTo(pwh,Self.Height-pwh);LineTo(Self.Width-pwh,pwh); end;
    end;
  end;
end;

procedure TXLinie.SetDirection(Value : LineDirectionType);
begin
  LineDirection:=Value;
  Invalidate;
end;

procedure TXLinie.SetPen(Value : TPen);
begin
  FPen.Assign(Value);
end;

procedure TXLinie.StyleChanged(Sender : TObject);
begin
  Invalidate;
end;

(* Methoden für Dreieck ===========================================*)
constructor TDreieck.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Self.Width:= 50;
  Self.Height:= 50;
  FSpitze:=25;
  FSPitzenSeite:=Rechts;
  FPen:=TPen.Create;
  FBrush:=TBrush.Create;
  FPen.OnChange:=StyleChanged;
  FBrush.OnChange:=StyleChanged;
end;

destructor TDreieck.Destroy;
begin
  FPen.Free;
  FBrush.Free;
  inherited Destroy;
end;

procedure TDreieck.Paint;
var
  pw,pwh : INTEGER;
  pts : ARRAY[0..2] of TPoint;
begin
  with Canvas do begin
    Pen:=FPen;
    Brush:=FBrush;
    pw:=Pen.Width;
    pwh:=pw div 2;
    if pwh=0 then pwh:=1;
    if Self.Width<pw THEN Self.Width:=pw;
    if Self.Height<pw THEN Self.Height:=pw;
    case FSpitzenSeite of
      Rechts: begin
        pts[0].X:=pwh;pts[0].y:=pwh;  { Links oben }
        pts[1].X:=Self.Width-pwh;pts[1].y:=FSpitze; { Nach rechts 1/2 unten }
        pts[2].X:=pwh;pts[2].y:=Self.Height-pwh;{ Nach links unten }
      end;
      Links:begin
        pts[0].X:=Self.Width-pwh;pts[0].y:=pwh;  { Rechts oben }
        pts[1].X:=pwh;pts[1].y:=FSpitze; { Nach links 1/2 unten }
        pts[2].X:=Self.Width-pwh;pts[2].y:=Self.Height-pwh;{ Nach rechts unten }
      end;
      Unten: begin
        pts[0].X:=pwh;pts[0].y:=pwh;  { Links oben }
        pts[1].X:=Self.Width-pwh;pts[1].y:=pwh; { rechts oben }
        pts[2].X:=Spitze;pts[2].y:=Self.Height-pwh;{ Nach 1/2 links unten }
      end;
      Oben: begin
        pts[0].X:=pwh;pts[0].y:=Self.Height-pwh;  { Links oben }
        pts[1].X:=Self.Width-pwh;pts[1].y:=Self.Height-pwh; { rechts unten }
        pts[2].X:=Spitze;pts[2].y:=pwh;{ Nach 1/2 links oben }
      end;
    end;
    Polygon(pts);
  end;
end;

procedure TDreieck.SetSpitzenSeite(Value : SpitzenSeiteType);
begin
  FSpitzenSeite:=Value;
  Invalidate;
end;

procedure TDreieck.SetSpitze(Value : integer);
begin
  FSpitze:=Value;
  Invalidate;
end;

procedure TDreieck.SetPen(Value : TPen);
begin
  FPen.Assign(Value);
end;

procedure TDreieck.SetBrush(Value : TBrush);
begin
  FBrush.Assign(Value);
end;

procedure TDreieck.StyleChanged(Sender : TObject);
begin
  Invalidate;
end;

(* Methoden für Schalter ========================================*)
constructor TSchalter.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Self.Width:= 50;
  Self.Height:= Self.Width*8 div 12;
  FDrehpunktSeite:=DPLinks;
  FPen:=TPen.Create;
  FBrush:=TBrush.Create;
  FBrush.color:=clBtnFace;
  FPen.OnChange:=StyleChanged;
  FBrush.OnChange:=StyleChanged;
end;

destructor TSchalter.Destroy;
begin
  FPen.Free;
  FBrush.Free;
  inherited Destroy;
end;

procedure TSchalter.Paint;
var
  pw,pwh,r : INTEGER;   { r=Radius der Kreischen }
  x1,y1,x2,y2: integer; { Koordinaten der Kreise }
  dx,dy : integer;      { Endpunktversatz }

begin
  with Canvas do begin
    Pen:=FPen;
    Brush:=FBrush;
    if (FDrehpunktSeite=DPRechts) or (FDrehpunktSeite=DPlinks) then begin
      r:=Self.Width div 10; { Kreischen }
      if r=0 then begin r:=2; Self.Width:=2*10; end;
      Self.Height:=Self.Width*8 div 12;
    end
    else begin
      r:=Self.Height div 10; { Kreischen }
      if r=0 then begin r:=2; Self.Height:=2*10; end;
      Self.Width:=Self.Height*8 div 12;
    end;
    case FDrehpunktSeite of
      DPRechts: begin
        x1:=Self.Width-r-Pen.Width; y1:=Self.Height div 2;  { Kreis 1 }
        x2:=r+Pen.Width;       y2:=Self.Height div 2;  { Kreis 2 }
        dx:=2*r;
        if FOeffnenNachLinks then dy:=Self.Height div 2 - Pen.Width
        else  dy:=-(Self.Height div 2)+Pen.Width;
      end;
      DPLinks:begin
        x1:=r+Pen.Width;       y1:=Self.Height div 2;  { Kreis 1 }
        x2:=Self.Width-r-Pen.Width; y2:=Self.Height div 2;  { Kreis 2 }
        dx:=-2*r;
        if FOeffnenNachLinks then dy:=-(Self.Height div 2)+Pen.Width
        else  dy:=Self.Height div 2-Pen.Width;
      end;
      DPUnten: begin
        x1:=Self.Width div 2; y1:=Self.Height-r-Pen.Width;  { Kreis 1 }
        x2:=Self.Width div 2; y2:=r+Pen.Width;         { Kreis 2 }
        dy:=2*r;
        if FOeffnenNachLinks then dx:=-(Self.Width div 2)+Pen.Width
        else dx:=Self.Width div 2-Pen.Width;
      end;
      DPOben: begin
        x1:=Self.Width div 2; y1:=r+Pen.Width;         { Kreis 1 }
        x2:=Self.Width div 2; y2:=Self.Height-r-Pen.Width;  { Kreis 2 }
        dy:=-2*r;
        if FOeffnenNachLinks then dx:=Self.Width div 2-Pen.Width
        else dx:=-(Self.Width div 2)+Pen.Width;
      end;
    end;
    MoveTo(x1,y1); { Drehpunkt }
    if FState then begin  { Geschlossen }
      LineTo(x2,y2);
    end
    else begin  { Offen }
      LineTo(x2+dx,y2+dy);
    end;
    Ellipse(x1-r,y1-r,x1+r,y1+r); { Drehpunkt }
    Ellipse(x2-r,y2-r,x2+r,y2+r); { Gegenpunkt }
  end;
end;


procedure TSchalter.SetPen(Value : TPen);
begin
  FPen.Assign(Value);
end;

procedure TSchalter.SetBrush(Value : TBrush);
begin
  FBrush.Assign(Value);
end;

procedure TSchalter.SetDrehpunktSeite(Value : DrehpunktSeiteType);
begin
  FDrehpunktSeite:=Value;
  Invalidate;
end;

procedure TSchalter.SetOeffnenNachLinks(Value : Boolean);
begin
  FOeffnenNachLinks:=Value;
  Invalidate;
end;

procedure TSchalter.SetState(Value : boolean);
begin
  FState:=Value;
  Invalidate;
  Change;
end;

procedure TSchalter.StyleChanged(Sender : TObject);
begin
  Invalidate;
end;

procedure TSchalter.Change;
begin
 if Assigned(FOnChange) then FOnChange(Self);
end;

(* Methoden für Umchalter ========================================*)
constructor TUmschalter.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Self.Width:= 50;
  Self.Height:= Self.Width*8 div 12;
  FDrehpunktSeite:=DPLinks;
  FPen:=TPen.Create;
  FBrush:=TBrush.Create;
  FBrush.color:=clBtnFace;
  FPen.OnChange:=StyleChanged;
  FBrush.OnChange:=StyleChanged;
end;

destructor TUmschalter.Destroy;
begin
  FPen.Free;
  FBrush.Free;
  inherited Destroy;
end;

procedure TUmschalter.Paint;
var
  r : INTEGER;   { r=Radius der Kreischen }
  x1,y1,x2,y2,x3,y3: integer; { Koordinaten der Kreise }

begin
  with Canvas do begin
    Pen:=FPen;
    Brush:=FBrush;
    if (FDrehpunktSeite=DPRechts) or (FDrehpunktSeite=DPlinks) then begin
      r:=Self.Width div 10; { Kreischen }
      if r=0 then begin r:=2; Self.Width:=2*10; end;
      Self.Height:=Self.Width*8 div 12;
    end
    else begin
      r:=Self.Height div 10; { Kreischen }
      if r=0 then begin r:=2; Self.Height:=2*10; end;
      Self.Width:=Self.Height*8 div 12;
    end;
    case FDrehpunktSeite of
      DPRechts: begin
        x1:=Self.Width-r-Pen.Width; y1:=Self.Height div 2;  { Kreis 1 }
        x2:=r+Pen.Width;       y2:=r+Pen.Width ;  { Kreis 2 }
        x3:=r+Pen.Width;       y3:=Self.Height-r-Pen.Width;  { Kreis 3 }
      end;
      DPLinks:begin
        x1:=r+Pen.Width;       y1:=Self.Height div 2;  { Kreis 1 }
        x2:=Self.Width-r-Pen.Width; y2:=Self.Height-r-Pen.Width;  { Kreis 2 }
        x3:=Self.Width-r-Pen.Width; y3:=r+Pen.Width;  { Kreis 3 }
      end;
      DPUnten: begin
        x1:=Self.Width div 2; y1:=Self.Height-r-Pen.Width;  { Kreis 1 }
        x2:=Self.Width-r-Pen.Width; y2:=r+Pen.Width;   { Kreis 2 }
        x3:=r+Pen.Width;       y3:=r+Pen.Width;   { Kreis 3 }
      end;
      DPOben: begin
        x1:=Self.Width div 2; y1:=r+Pen.Width;         { Kreis 1 }
        x2:=r+Pen.Width; y2:=Self.Height-r-Pen.Width;  { Kreis 2 }
        x3:=Self.Width-r-Pen.Width; y3:=Self.Height-r-Pen.Width;   { Kreis 3 }
      end;
    end;
    MoveTo(x1,y1); { Drehpunkt }
    if FState then begin  { Geschlossen }
      LineTo(x2,y2);
    end
    else begin  { Offen }
      LineTo(x3,y3);
    end;
    Ellipse(x1-r,y1-r,x1+r,y1+r); { Drehpunkt }
    Ellipse(x2-r,y2-r,x2+r,y2+r); { 1. Gegenpunkt }
    Ellipse(x3-r,y3-r,x3+r,y3+r); { 2. Gegenpunkt }
  end;
end;


procedure TUmschalter.SetPen(Value : TPen);
begin
  FPen.Assign(Value);
end;

procedure TUmschalter.SetBrush(Value : TBrush);
begin
  FBrush.Assign(Value);
end;

procedure TUmschalter.SetDrehpunktSeite(Value : DrehpunktSeiteType);
begin
  FDrehpunktSeite:=Value;
  Invalidate;
end;

procedure TUmschalter.SetState(Value : boolean);
begin
  FState:=Value;
  Invalidate;
  Change;
end;

procedure TUmschalter.StyleChanged(Sender : TObject);
begin
  Invalidate;
end;

procedure TUmschalter.Change;
begin
 if Assigned(FOnChange) then FOnChange(Self);
end;

end.
