unit Uhr;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

uses
{$IFnDEF FPC}
  Windows,
{$ELSE}
{$IFnDEF FPC}
  Windows,
{$ELSE}
{$ENDIF}
  LCLIntf, LCLType, LMessages,
{$ENDIF}
  Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, Windows,
  StdCtrls;


type
  TUhr = class(TGraphicControl)
  private
    { Private-Deklarationen }
    FPen : TPen;      { Kreis + Teilstriche }
    FBrush : TBrush;  { Hintergrung }
    FZeigerfarbe : TColor; { Zeigerfarbe }
    FFont : TFont;    { Für Stunden }
    FZeit : TDateTime;{ Diese Zeit wird angezeigt }
    procedure SetBrush(Value : TBrush);
    procedure SetZeigerfarbe(Value : TColor);
    procedure SetZeit(Value : TDateTime);
    procedure SetPen(Value : TPen);
    procedure SetFont(Value : TFont);
  protected
    { Protected-Deklarationen }
    procedure Paint; override;
  public
    { Public-Deklarationen }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published-Deklarationen }
    property Height default 200;
    property Brush: TBrush read FBrush write SetBrush;
    property Zeigerfarbe: TColor read FZeigerfarbe write SetZeigerfarbe;
    property Zeit : TDateTime read FZeit write SetZeit;
    property Pen: TPen read FPen write SetPen;
    property Font: TFont read FFont write SetFont;
    property DragCursor;
    property DragMode;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    procedure StyleChanged(Sender : TObject);
    procedure BrushStyleChanged(Sender : TObject);
  end;

procedure Register;

implementation
(*
{$R TUhr.lrs }
*)

(* Methoden für TUhr =========================================*)
function Schattenfarbe(c : TColor): TColor;
var
  r,b,g : byte;
begin
  { Schattenfarbe bestimmen }
  r:=LongInt(c) and $FF;
  g:=(LongInt(c) and $FF00) shr 8;
  b:=(LongInt(c) and $FF0000) shr 16;
  { Alle Farbkompoenten ein bisschen dunkler=kleiner }
  r:=r*20 div 28;
  b:=b*20 div 28;
  g:=g*20 div 28;
  Schattenfarbe:=TColor((b shl 16)+(g shl 8)+r); { Schattenfarbe }
end;

constructor TUhr.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Self.Width:=200;
  Self.Height:=Width;
  FBrush:=TBrush.Create;  { Hintergrund }
  FBrush.OnChange:=BrushStyleChanged;
  FFont:=TFont.Create;
  FFont.Size:=Width div 15;
  FFont.OnChange:=StyleChanged;
  FPen:=TPen.Create;
  FPen.OnChange:=StyleChanged;
  FZeit:=0;
end;

destructor TUhr.Destroy;
begin
  FPen.Free;
  inherited Destroy;
end;

procedure TUhr.Paint;
const
  bm = pi/180;
var
  xm,ym,r,w, tsl,tsll : integer;
  x,y:integer;

begin
  with Canvas do begin
    Pen:=FPen;     { Kreis }
    Brush:=FBrush; { Hintergrund }
    Font:=FFont;
    xm:=Self.Width div 2;
    ym:=Self.Height div 2;
    if Self.Height<Self.Width then r:=ym-Pen.Width  { Radius des Kreises }
    else r:=xm-Pen.Width;
    Ellipse(xm-r,ym-r,xm+r,ym+r); { Kreis }
    tsl:=r div 20;    { Länge Teilstrich }
    tsll:=2*tsl;      { Länge 5-Min-Teilstrich }
    w:=0;
    while w<360 do begin { Teilstriche zeichnen }
      tsl:=r div 20;     { Teilstrichlänge }
      if (w mod 5) = 0 then  tsl:=tsll; { 5-min-Teilstrichlänge }
      x:=round((r-tsl)*sin(bm*w));y:=-round((r-tsl)*cos(bm*w));
      MoveTo(xm+x,ym+y);
      x:=round(r*sin(bm*w));y:=-round(r*cos(bm*w));
      LineTo(xm+x,ym+y);
      w:=w+6; { alle Minuten 1 Teilstrich }
    end;
    Zeit:=Zeit; { Damit Zeiger neu angezeigt werden }
  end; { with }
end;

procedure TUhr.SetZeit(Value : TDateTime);
const
  bm = pi/180;
var
  xm, ym, r, tsl,tsll : integer;
  std, min, sec, ms : word; { Stunde, Minute, Sekunde, ms-Sec }
  lstd, lmin, lsec : word; { Zeigerlängen }
  wstd, wmin, wsec : integer; { Winkel für die Zeiger }
  ts : TSize;
  scol : TColor; { Schattenfarbe }

procedure ZeigZeiger(Wi,lz,db,ds:integer;col:TColor);
var
  pts : ARRAY[0..3] of TPoint;

begin
  Canvas.Brush.Color:=col;
  Canvas.Pen.Color:=col;
  pts[0].X:=xm+round(lz*sin(bm*wi))+ds; { Spitze}
  pts[0].Y:=ym-round(lz*cos(bm*wi))+ds;
  pts[1].X:=xm-round(db*sin(bm*(wi+90)))+ds; { linke Ecke }
  pts[1].Y:=ym+round(db*cos(bm*(wi+90)))+ds;
  pts[2].X:=xm-round((lz div 10)*sin(bm*wi))+ds;   { Spitze kurz }
  pts[2].Y:=ym+round((lz div 10)*cos(bm*wi))+ds;
  pts[3].X:=xm-round(db*sin(bm*(wi-90)))+ds; { rechte Ecke }
  pts[3].Y:=ym+round(db*cos(bm*(wi-90)))+ds;
  Canvas.Polygon(pts);
end;

begin
  with Canvas do begin
    xm:=Self.Width div 2;
    ym:=Self.Height div 2;
    Font:=FFont;
    if Self.Height<Self.Width then r:=ym-Pen.Width  { Radius des Kreises }
    else r:=xm-Pen.Width;
    lstd:=r*20 div 35;  { Länge Stundenzeiger }
    lmin:=r*20 div 28;  { Länge Minutenzeiger }
    lsec:=r*20 div 24;  { Länge Sekundenzeiger }
    tsl:=r div 20;      { Länge Teilstrich }
    tsll:=2*tsl;        { Länge 5 min-Teilstrich }

    {alte Zeit, Zeiger weg }
    DecodeTime(FZeit,std,min,sec,ms);
    wstd:=((std mod 12)*60+min) div 2; {Winkel des Stundenzeigers }
    wmin:=min*6;  {Winkel des Minutenzeigers }
    wsec:=sec*6;  {Winkel des Sekundenzeigers }
    ZeigZeiger(wsec,lsec,1,2,FBrush.Color);
    ZeigZeiger(wsec,lsec,1,0,FBrush.Color);
    ZeigZeiger(wmin,lmin,2,2,FBrush.Color);
    ZeigZeiger(wmin,lmin,2,0,FBrush.Color);
    ZeigZeiger(wstd,lstd,4,2,FBrush.Color);
    ZeigZeiger(wstd,lstd,4,0,FBrush.Color);

    { Zahlen neu zeichnen }
    Brush:=FBrush; { Hintergrund }
    Pen:=FPen;
    ts:=TextExtent('12');
    TextOut(xm-ts.cx div 2,ym-r+tsll+7,'12');
    ts:=TextExtent('3');
    TextOut(xm+r-tsll-ts.cx-7,ym-ts.cy div 2,'3');
    ts:=TextExtent('6');
    TextOut(xm-ts.cx div 2,ym+r-tsll-ts.cy-7,'6');
    ts:=TextExtent('9');
    TextOut(xm-r+tsll+7,ym-ts.cy div 2,'9');

    { Neue Zeit anzeigen }
    FZeit:=Value;
    DecodeTime(FZeit,std,min,sec,ms);
    wstd:=((std mod 12)*60+min) div 2;
    wmin:=min*6;
    wsec:=sec*6;
    scol:=Schattenfarbe(Brush.Color);
    ZeigZeiger(wstd,lstd,4,2,scol); { Stundenzeiger Schatten }
    ZeigZeiger(wstd,lstd,4,0,FZeigerfarbe); { Stundenzeiger selbst }
    ZeigZeiger(wmin,lmin,2,2,scol); { Minutenzeiger }
    ZeigZeiger(wmin,lmin,2,0,FZeigerfarbe);
    ZeigZeiger(wsec,lsec,1,2,scol); { Sekundenzeiger }
    ZeigZeiger(wsec,lsec,1,0,FZeigerfarbe);
  end; { with }
end;

procedure TUhr.SetBrush(Value : TBrush);
begin
  FBrush:=Value;  { Hintergrund }
  Invalidate;
end;

procedure TUhr.SetFont(Value : TFont);
begin
  FFont:=Value;
  Invalidate;
end;

procedure TUhr.SetPen(Value : TPen);
begin
  FPen:=Value;
  Invalidate;
end;

procedure TUhr.SetZeigerfarbe(Value : TColor);
begin
  FZeigerfarbe:=Value;  { Farbe der Zeiger}
  Invalidate;
end;

procedure TUhr.BrushStyleChanged(Sender : TObject);
begin
  invalidate;
end;

procedure TUhr.StyleChanged(Sender : TObject);
begin
  Invalidate;
end;

procedure Register;
begin
  RegisterComponents('Beispiele', [TUhr]);
end;

end.
