(* Komponenten Windrose, Massstab selbst entwickelt *)

unit Windrose;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

uses
{$IFnDEF FPC}
  WinTypes, WinProcs,
{$ELSE}
{$IFnDEF FPC}
  Windows,
{$ELSE}
{$ENDIF}
  LCLIntf, LCLType, LMessages,
{$ENDIF}
  SysUtils, Messages, Classes, Graphics, Controls, Windows,
  Forms, Dialogs, ExtCtrls;


type
  StrichRichtungType = (Links,rechts,oben,unten);
  TMassStab = class(TGraphicControl)
  private
    { Private-Deklarationen }
    FStrichRichtung : StrichRichtungType;
    FPen : TPen;
    FBrush : TBrush;
    FFont : TFont;
    FMinValue : integer;
    FMaxValue : integer;
    FRand : integer;
    FStep : integer;
    FHauptInterval : integer;
    FNebenInterval : integer;
    FTeilStrichLaenge : integer;
    procedure SetStrichRichtung(Value : StrichRichtungType);
    procedure SetPen(Value : TPen);
    procedure SetBrush(Value : TBrush);
    procedure SetFont(Value : TFont);
    procedure SetMinValue(Value : integer);
    procedure SetMaxValue(Value : integer);
    procedure SetRand(Value : integer);
    procedure SetStep(Value : integer);
    procedure SetHauptInterval(Value : integer);
    procedure SetNebenInterval(Value : integer);
    procedure SetTeilStrichLaenge(Value : integer);
  protected
    { Protected-Deklarationen }
    procedure Paint; override;
  public
    { Public-Deklarationen }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published-Deklarationen }
    property StrichRichtung : StrichRichtungType read  FStrichRichtung write SetStrichRichtung;
    property Height default 200;
    property Width default 20;
    property Brush: TBrush read FBrush write SetBrush;
    property MinValue : integer read FMinValue write SetMinValue;
    property MaxValue : integer read FMaxValue write SetMaxValue;
    property Rand : integer read FRand write SetRand;
    property Step : integer read FStep write SetStep ;
    property HauptInterval : integer read FHauptInterval write SetHauptInterval ;
    property NebenInterval : integer read FNebenInterval write SetNebenInterval ;
    property TeilStrichLaenge : integer read FTeilStrichLaenge write SetTeilStrichLaenge ;
    property Pen: TPen read FPen write SetPen;
    property Font: TFont read FFont write SetFont;
    property DragCursor;
    property DragMode;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    procedure StyleChanged(Sender : TObject);
  end;

  TOnChange = procedure(Sender: TObject;AktGrad : integer) of object;

  TWindrose = class(TGraphicControl)
  private
    { Private-Deklarationen }
    FOnChange: TOnChange;
    FPen : TPen;
    FBrush : TBrush;
    FWindFahnenfarbe : TColor;
    FFont : TFont;
    FGrad : integer;
    procedure SetPen(Value : TPen);
    procedure SetBrush(Value : TBrush);
    procedure SetWindFahnenfarbe(Value : TColor);
    procedure SetFont(Value : TFont);
    procedure SetGrad(Value : integer);
  protected
    { Protected-Deklarationen }
    procedure Paint; override;
    procedure DoOnChange(Sender: TObject;AktGrad : integer);dynamic;
  public
    { Public-Deklarationen }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published-Deklarationen }
    property OnChange: TOnChange read FOnChange write FOnChange;
    property Height default 200;
    property Width default 200;
    property Brush: TBrush read FBrush write SetBrush;
    property WindFahnenfarbe: TColor read FWindFahnenfarbe write SetWindFahnenfarbe;
    property Grad : integer read FGrad write SetGrad;
    property Pen: TPen read FPen write SetPen;
    property Font: TFont read FFont write SetFont;
    property DragCursor;
    property DragMode;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    procedure StyleChanged(Sender : TObject);
  end;


procedure Register;

implementation
(*
{$R TWindrose.lrs }
{$R TMassStab.lrs }
*)

procedure Register;
begin
  RegisterComponents('Beispiele', [TMassStab]);
  RegisterComponents('Beispiele', [TWindrose]);
end;

(* Methoden für TMassStab =========================================*)
constructor TMassStab.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Self.Width:= 300;
  Self.Height:= 40;
  FFont:=TFont.Create;
  FFont.OnChange:=StyleChanged;
  FBrush:=TBrush.Create;
  FBrush.OnChange:=StyleChanged;
  FBrush.color:=clBtnFace;
  FMinValue:=-10;
  FMaxValue:=30;
  FStrichRichtung:=oben;
  FStep:=1;
  FRand:=10;
  FNebenInterval:=5;
  FHauptInterval:=10;
  FTeilStrichLaenge:=5;
  FPen:=TPen.Create;
  FPen.OnChange:=StyleChanged;
end;

destructor TMassStab.Destroy;
begin
  FPen.Free;
  inherited Destroy;
end;

procedure TMassStab.Paint;
var
  xmin, xmax, xhsmin, xhsmax, xakt, dx, dxtext : INTEGER; { Hori Pixel der Teilstriche }
  ymin, ymax, yhsmin, yhsmax, yakt, dy, dytext : INTEGER; { Verti Pixel des Hauptstrichs }
  n : integer;
  ts : TSize;
  WertString : String [10];
  rect: TRect;
begin
  with Canvas do begin
    Pen:=FPen;
    Brush:=FBrush;
    Font:=FFont;
    rect.Left:=0;rect.Top:=0;rect.Right:=Self.Width;rect.Bottom:=Self.Height;
    FillRect(rect);
    case StrichRichtung of
      links,rechts: begin
        if StrichRichtung=links then begin
          xmin:=0; xmax:=TeilStrichLaenge;
        end
        else begin
          xmin:=Self.Width-1;  xmax:=Self.Width-TeilStrichLaenge;
        end;
        dx:=(xmax-xmin) div 2;
        yhsmin:=Self.Height-1-Rand; yhsmax:= Rand;
        dy:=yhsmax-yhsmin;
        MoveTo(xmin,yhsmin);LineTo(xmin,yhsmax);{ Hauptstrich}
        n:= MinValue;
        while  n<=MaxValue do begin
          xakt:=xmax; { Normale Teilstriche }
          yakt:= yhsmin+dy*(n-MinValue) div (MaxValue-Minvalue );
          if ((n mod NebenInterval)=0) then xakt:=xakt+dx;
          if ((n mod HauptInterval)=0) then begin
            xakt:=xakt+dx;
            WertString:=IntToStr(n);
            ts:=TextExtent(WertString);
            if StrichRichtung=links then dxtext:=2
            else dxtext:=-ts.cx;
            TextOut(xakt+dxtext,yakt-ts.cy div 2,WertString);
          end;
          MoveTo(xmin,yakt);
          LineTo(xakt,yakt);
          n:=n+step;
        end; { while }
      end; { links, rechts }
      oben, unten: begin
        if StrichRichtung=unten then begin
          ymin:=0; ymax:=TeilStrichLaenge;
        end
        else begin
          ymin:=Self.Height-1; ymax:=Self.Height-TeilStrichLaenge;
        end;
        dy:=(ymax-ymin) div 2;
        xhsmin:=Rand; xhsmax:= Self.Width-1-Rand;
        dx:=xhsmax-xhsmin;
        MoveTo(xhsmin,ymin);LineTo(xhsmax,ymin);{ Hauptstrich}
        n:= MinValue;
        while  n<=MaxValue do begin
          yakt:=ymax; { Normale Teilstriche }
          xakt:= xhsmin+dx*(n-MinValue) div (MaxValue-Minvalue );
          if ((n mod NebenInterval)=0) then yakt:=yakt+dy;
          if ((n mod HauptInterval)=0) then begin
            yakt:=yakt+dy;
            WertString:=IntToStr(n);
            ts:=TextExtent(WertString);
            if StrichRichtung=unten then dytext:=2
            else dytext:=-ts.cy;
            TextOut(xakt-ts.cx div 2,yakt+dytext,WertString);
          end;
          MoveTo(xakt,ymin);
          LineTo(xakt,yakt);
          n:=n+step;
        end; { while }
      end;
    end; { case }
  end;
end;

procedure TMassStab.SetStrichRichtung(Value : StrichRichtungType);
begin
  FStrichRichtung:=Value;
  Invalidate;
end;

procedure TMassStab.SetMinValue(Value : integer);
begin
  FMinValue:=Value;
  Invalidate;
end;

procedure TMassStab.SetMaxValue(Value : integer);
begin
  FMaxValue:=Value;
  Invalidate;
end;

procedure TMassStab.SetRand(Value : integer);
begin
  FRand:=Value;
  Invalidate;
end;

procedure TMassStab.SetStep(Value : integer);
begin
  FStep:=Value;
  Invalidate;
end;

procedure TMassStab.SetHauptInterval(Value : integer);
begin
  FHauptInterval:=Value;
  Invalidate;
end;

procedure TMassStab.SetNebenInterval(Value : integer);
begin
  FNebenInterval:=Value;
  Invalidate;
end;

procedure TMassStab.SetTeilStrichLaenge(Value : integer);
begin
  FTeilStrichLaenge:=Value;
  Invalidate;
end;

procedure TMassStab.SetPen(Value : TPen);
begin
  FPen.Assign(Value);
end;

procedure TMassStab.SetBrush(Value : TBrush);
begin
  FBrush.Assign(Value); { Farbe Hintergrung }
end;

procedure TMassStab.SetFont(Value : TFont);
begin
  FFont.Assign(Value);
end;

procedure TMassStab.StyleChanged(Sender : TObject);
begin
  Invalidate;
end;


(* Methoden für TWindrose =========================================*)
function Schattenfarbe(c : TColor): TColor;
var
  r,b,g : byte;
begin
  { Schattenfarbe bestimmen }
  r:=LongInt(c) and $FF;
  g:=(LongInt(c) and $FF00) shr 8;
  b:=(LongInt(c) and $FF0000) shr 16;
  { Alle Farbkompoenten ein bisschen dunkler=kleiner }
  r:=r*20 div 28;
  b:=b*20 div 28;
  g:=g*20 div 28;
  Schattenfarbe:=TColor((b shl 16)+(g shl 8)+r); { Schattenfarbe }
end;

constructor TWindrose.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Self.Width:= 160;
  Self.Height:=160;
  FBrush:=TBrush.Create;
  FBrush.OnChange:=StyleChanged;
  FFont:=TFont.Create;
  FFont.OnChange:=StyleChanged;
  FPen:=TPen.Create;
  FPen.OnChange:=StyleChanged;
  FGrad:=0;
end;

procedure TWindrose.DoOnChange(Sender: TObject;AktGrad:integer);
begin
  if Assigned(OnChange) then OnChange(Self,AktGrad);
end;

destructor TWindrose.Destroy;
begin
  FPen.Free;
  inherited Destroy;
end;

procedure TWindrose.Paint;

var
  xm,ym,r : integer;

begin
  with Canvas do begin
    Pen:=FPen;
    Brush:=FBrush;
    Font:=FFont;
    xm:=Self.Width div 2;
    ym:=Self.Height div 2;
    if Self.Width>Self.Height then r:=Self.Height div 2 - Pen.Width
    else r:=Self.Width div 2 - Pen.Width;
    Ellipse(xm-r,ym-r,xm+r,ym+r);
    Grad:=Grad; { Damit Windfahne angezeigt wird }
  end; { with }
end;

procedure TWindrose.SetGrad(Value : integer);
const
  bm = pi/180;
var
  x, y, xm, ym, r, lz, w ,tsl: integer;
  gr : word; { Stunde, Minute, Sekunde, ms-Sec }
  ts : TSize;
  scol : TColor; { Schattenfarbe }

procedure ZeigZeiger(Wi,lz,ds:integer;col:TColor);
var
  pts : ARRAY[0..2] of TPoint;

begin
  Canvas.Brush.Color:=col;
  Canvas.Pen.Color:=col;
  pts[0].X:=xm+trunc((r-tsl )*sin(bm*(Grad-4)))+ds;
  pts[0].Y:=ym-trunc((r-tsl )*cos(bm*(Grad-4)))+ds;
  pts[1].X:=xm+trunc((r-tsl )*sin(bm*(Grad+4)))+ds;
  pts[1].Y:=ym-trunc((r-tsl )*cos(bm*(Grad+4)))+ds;
  pts[2].X:=xm+trunc((r-2*lz)*sin(bm*Grad))+ds;
  pts[2].Y:=ym-trunc((r-2*lz)*cos(bm*Grad))+ds;
  Canvas.Polygon(pts);
end;

begin
  with Canvas do begin
    Brush:=FBrush; { Hintergrund }
    Pen:=FPen;
    Font:=FFont;
    xm:=Self.Width div 2;
    ym:=Self.Height div 2;
    if Self.Width>Self.Height then r:=Self.Height div 2 - Pen.Width
    else r:=Self.Width div 2 - Pen.Width;
    lz:=r div 5;
    tsl:=r div 10;
    {alte Zeit, Zeiger weg }
    ZeigZeiger(FGrad,lz,2,FBrush.Color);
    ZeigZeiger(FGrad,lz,0,FBrush.Color);

    Brush:=FBrush; { Hintergrund }
    Pen:=FPen;
    w:=0;
    while w<360 do begin { Teilstriche zeichnen }
      x:=round((r-tsl)*sin(bm*w));y:=-round((r-tsl)*cos(bm*w));
      MoveTo(xm+x,ym+y);
      x:=round(r*sin(bm*w));y:=-round(r*cos(bm*w));
      LineTo(xm+x,ym+y);
      w:=w+10;
    end;
    { Zahlen neu zeichnen }
    ts:=TextExtent('Nord 0°');
    TextOut(xm-ts.cx div 2,ym-r+tsl*2,'Nord 0°');
    ts:=TextExtent('Ost');
    TextOut(xm+r-tsl*2-ts.cx+2,ym-ts.cy,'Ost');
    ts:=TextExtent('90°');
    TextOut(xm+r-tsl*2-ts.cx+2,ym,'90°');
    ts:=TextExtent('Sud 180°');
    TextOut(xm-ts.cx div 2,ym+r-tsl*2-ts.cy,'Süd 180°');
    ts:=TextExtent('West');
    TextOut(xm-r+tsl*2,ym-ts.cy,'West');
    ts:=TextExtent('270°');
    TextOut(xm-r+tsl*2,ym,'270°');

    { Neue Zeit anzeigen }
    FGrad:=Value;
    scol:=Schattenfarbe(Brush.Color);
    ZeigZeiger(FGrad,lz,2,scol); {  Schatten }
    ZeigZeiger(FGrad,lz,0,FWindfahnenfarbe); { Windfahne }
    DoOnChange(Self,FGrad);
  end; { with }
end;

procedure TWindrose.SetPen(Value : TPen);
begin
  FPen.Assign(Value);
end;

procedure TWindrose.SetBrush(Value : TBrush);
begin
  FBrush.Assign(Value);  { Farbe im Kreis }
end;

procedure TWindrose.SetWindFahnenfarbe(Value : TColor);
begin
  FWindFahnenfarbe:=Value; { Farbe der windfahne }
  Invalidate;
end;

procedure TWindrose.SetFont(Value : TFont);
begin
  FFont.Assign(Value);
end;

procedure TWindrose.StyleChanged(Sender : TObject);
begin
  Invalidate;
end;




end.
